%==========================================================================
% function for setting up the PKS Filter
%
% Inputs:
% Order - Number of polynominal factors a_n  (a_0 + a_1*s + a_2*s^2 + ..)
% shiftIncrement - step size for state transition (commonly 1 in discrete systems)
% leftWindow - number of points left hand site of smoothing point
% rightWindow - number of points right hand site of smoothing point
% lambda - factor for exponential forgetting (lambda = 1-1/FilterLength)
% Outputs:
% calA - state transition matrix in the statistical model:
%
% xApos(t)     = calA x(t-1) (for random walk calA = I)
% x(t)         = xApos(t)  + v
% d(t)         = calC x(t) + w
%
% with calC    = u(t)
% with Cov(v)  = R
% with Cov(w)  = Q
%
% K - Kalman gain, time invariant, size (n,1)
% u - input data, time invariant, size (1,n)
% uSmooth - input data for smooth filter output
% compute the filter output with:
% smoothSignal = uSmooth * X;
% firstDerivateSignal = (0:1:length(X)-1) * ([0,uSmooth(1:end-1)]'.*X);
%
function [calA, u, uSmooth] = fcn_PKSFilterInit(Order, shiftIncrement, ...
    leftWindow, rightWindow)
%
FilterLength = leftWindow + rightWindow + 1;
Koefficients = Order + 1;
u = zeros(1 , Koefficients); %init values
uSmooth = zeros(1 , Koefficients); %init values
for i=1:Koefficients
    u(i) = FilterLength^(i-1);
    uSmooth(i) = (leftWindow +1)^(i-1);
end   
calA = fcn_calA(Koefficients, shiftIncrement); %compute state transition matrix
end % of function==========================================================
%
%==========================================================================
% function to compute the state transition matrix calA in the statistical
% model
% statistical model:
% xApos(t)     = calA x(t-1) (for random walk calA = I)
% x(t)         = xApos(t)  + v
% d(t)         = calC x(t) + w
%
% with calC    = u(t)
% with Cov(v)  = R
% with Cov(w)  = Q
%
% Inputs:
% N : Number of polynominal factors a_n  (a_0 + a_1*s + a_2*s^2 + ..)
% ds : Shift length
%
% Outputs:
% calA : Shift matrix for a polynominal function 
% @Felix Bleimund 2013 KIT
%
function calA = fcn_calA(N, ds)
%#codegen
calA = zeros(N);
calA(1, :) = ones(1,N);
% Derived in hand written notes from 2013-05-16 
% "2013-05-19_notes to polynominal shif matrix.pdf"
for i = 2:N
    for j = 2:N
        calA(i,j) = (calA(i-1,j-1) +  calA(i,j-1)) ;
    end
end
%
for i = 1:N
    for j=1:N
        calA(i,j) = calA(i,j)*ds^(j-i);
    end
end

end % of function==========================================================